"""Ariadne 内置的 Dispatcher"""

from graia.broadcast.entities.dispatcher import BaseDispatcher as AbstractDispatcher
from graia.broadcast.interfaces.dispatcher import DispatcherInterface

from .message.chain import MessageChain
from .message.element import Source
from .typing import generic_isinstance, generic_issubclass


class MessageChainDispatcher(AbstractDispatcher):
    """从 MessageEvent 提取 MessageChain 的 Dispatcher"""

    @staticmethod
    async def catch(interface: DispatcherInterface):
        from .event.message import ActiveMessage, MessageEvent

        if isinstance(interface.event, (MessageEvent, ActiveMessage)):
            if generic_issubclass(MessageChain, interface.annotation):
                return interface.event.messageChain


class ContextDispatcher(AbstractDispatcher):
    """提取上下文的 Dispatcher"""

    @staticmethod
    async def catch(interface: DispatcherInterface):
        from . import get_running

        if generic_isinstance(interface.event, interface.annotation):
            return interface.event

        return get_running(interface.annotation)


class SourceDispatcher(AbstractDispatcher):
    """提取 MessageEvent 消息链 Source 元素的 Dispatcher"""

    @staticmethod
    async def catch(interface: DispatcherInterface):
        from .event.message import ActiveMessage, MessageEvent

        if isinstance(interface.event, (MessageEvent, ActiveMessage)):
            if generic_issubclass(Source, interface.annotation):
                return interface.event.messageChain.getFirst(Source)


class SenderDispatcher(AbstractDispatcher):
    """
    从 MessageEvent 提取 sender 的 Dispatcher.
    """

    @staticmethod
    async def catch(interface: DispatcherInterface):
        from .event.message import MessageEvent

        if isinstance(interface.event, MessageEvent):
            try:
                if generic_isinstance(interface.event.sender, interface.annotation):
                    return interface.event.sender
            except TypeError:
                pass


class SubjectDispatcher(AbstractDispatcher):
    """从 ActiveMessage 提取 subject 的 Dispatcher"""

    @staticmethod
    async def catch(interface: DispatcherInterface):
        from .event.message import ActiveMessage

        if isinstance(interface.event, ActiveMessage):
            if generic_issubclass(interface.annotation, interface.event.subject):
                return interface.event.subject


class BaseDispatcher(AbstractDispatcher):
    """空 Dispatcher"""

    @staticmethod
    async def catch(*_):
        pass



"""Alconna 的简单封装"""
import traceback
from typing import TYPE_CHECKING, Callable, Optional

from arclet.alconna import (
    Alconna,
    Arpamar,
    MessageChain,
    NonTextElement,
    ParamsUnmatched,
    compile,
    require_help_send_action,
)
from graia.broadcast.entities.dispatcher import BaseDispatcher
from graia.broadcast.entities.event import Dispatchable
from graia.broadcast.exceptions import ExecutionStop
from graia.broadcast.interfaces.dispatcher import DispatcherInterface

from ... import get_running
from ...app import Ariadne
from ...dispatcher import ContextDispatcher
from ...event.message import GroupMessage, MessageEvent
from ...util import resolve_dispatchers_mixin
from ..chain import MessageChain as GraiaMessageChain

if TYPE_CHECKING:
    ArpamarProperty = type("ArpamarProperty", (str, MessageChain, NonTextElement), {})
else:
    ArpamarProperty = type("ArpamarProperty", (), {})


class AlconnaHelpMessageDispatcher(BaseDispatcher):
    mixin = [ContextDispatcher]

    def __init__(self, alconna: Alconna, help_string: str, source_event: MessageEvent):
        self.command = alconna
        self.help_string = help_string
        self.source_event = source_event

    async def catch(self, interface: "DispatcherInterface"):
        if interface.name == "help_string" and interface.annotation == str:
            return self.help_string
        if interface.annotation == Alconna:
            return self.command
        if issubclass(interface.annotation, MessageEvent) or interface.annotation == MessageEvent:
            return self.source_event


class AlconnaHelpMessage(Dispatchable):
    """
    Alconna帮助信息发送事件

    如果触发的某个命令的帮助选项, 当AlconnaDisptcher的reply_help为False时, 会发送该事件
    """

    command: Alconna
    """命令"""

    help_string: str
    """帮助信息"""

    source_event: MessageEvent
    """来源事件"""


class AlconnaDispatcher(BaseDispatcher):
    """
    Alconna的调度器形式
    """

    def __init__(
        self,
        *,
        alconna: Alconna,
        reply_help: bool = False,
        skip_for_unmatch: bool = True,
        help_handler: Optional[Callable[[str], GraiaMessageChain]] = None,
    ):
        """
        构造 Alconna调度器

        Args:
            alconna (Alconna): Alconna实例
            reply_help (bool): 是否自助回复帮助信息给指令的发起者. 当为 False 时, 会广播一个'AlconnaHelpMessage'事件以交给用户处理帮助信息.
            skip_for_unmatch (bool): 当指令匹配失败时是否跳过对应的事件监听器, 默认为 True
        """
        super().__init__()
        self.analyser = compile(alconna)
        self.reply_help = reply_help
        self.skip_for_unmatch = skip_for_unmatch
        self.help_handler = help_handler or (lambda x: GraiaMessageChain.create(x))

    help_handler: Callable[[str], GraiaMessageChain]

    async def beforeExecution(self, interface: "DispatcherInterface[MessageEvent]"):
        """预处理消息链并存入 local_storage"""
        event: MessageEvent = interface.event

        if self.reply_help:
            app: Ariadne = get_running()

            async def _send_help_string(help_string: str):
                message = self.help_handler(help_string)
                if not isinstance(message, GraiaMessageChain):
                    message = GraiaMessageChain.create(message)
                if isinstance(event, GroupMessage):
                    await app.sendGroupMessage(event.sender.group, message)
                else:
                    await app.sendMessage(event.sender, message)

            require_help_send_action(_send_help_string, self.analyser.alconna.name)
        else:

            async def _post_help(help_string: str):
                dispatchers = resolve_dispatchers_mixin(
                    [
                        AlconnaHelpMessageDispatcher(self.analyser.alconna, help_string, event),
                        event.Dispatcher,
                    ]
                )
                for listener in interface.broadcast.default_listener_generator(AlconnaHelpMessage):
                    await interface.broadcast.Executor(listener, dispatchers=dispatchers)

            require_help_send_action(_post_help, self.analyser.alconna.name)

        local_storage = interface.local_storage
        chain: GraiaMessageChain = await interface.lookup_param("message_chain", GraiaMessageChain, None)
        try:
            result = self.analyser.analyse(chain)
        except ParamsUnmatched:
            traceback.print_exc()
            raise ExecutionStop
        if not result.matched and self.skip_for_unmatch:
            raise ExecutionStop
        local_storage["arpamar"] = result

    async def catch(self, interface: DispatcherInterface[MessageEvent]):
        local_storage = interface.local_storage
        arpamar: Arpamar = local_storage["arpamar"]
        if issubclass(interface.annotation, Arpamar):
            return arpamar
        if issubclass(interface.annotation, Alconna):
            return self.analyser.alconna
        if isinstance(interface.annotation, dict) and arpamar.options.get(interface.name):
            return arpamar.options[interface.name]
        if interface.name in arpamar.all_matched_args:
            if isinstance(arpamar.all_matched_args[interface.name], interface.annotation):
                return arpamar.all_matched_args[interface.name]
        if issubclass(interface.annotation, ArpamarProperty):
            return arpamar.get(interface.name)



from asyncio import Future
from typing import Any, Dict, Type, Union

from ..exception import (
    AccountMuted,
    AccountNotFound,
    InvalidArgument,
    InvalidSession,
    InvalidVerifyKey,
    MessageTooLong,
    RemoteException,
    UnknownError,
    UnknownTarget,
    UnVerifiedSession,
)


class SyncIDManager:
    def __init__(self) -> None:
        self.id_map: Dict[int, Future] = {0: ...}

    def allocate(self, fut: Future) -> int:
        """分配一个新的 Sync ID, 并将其与指定的 Future 关联.

        Returns:
            int: 生成的 Sync ID. 注意完成后使用 free() 方法标记本 Sync ID.
        """
        new_id = max(self.id_map) + 1
        self.id_map[new_id] = fut
        return new_id

    def free(self, sync_id: int, result: Any) -> bool:
        """标记一个 Sync ID 的任务完成. 本 Sync ID 随后可被复用.

        Args:
            sync_id (int): 标记的 Sync ID.
            result (Any): 任务结果
        Returns:
            bool: 是否成功标记.
        """
        if sync_id in self.id_map:
            fut = self.id_map.pop(sync_id)
            if isinstance(result, Exception):
                fut.set_exception(result)
            else:
                fut.set_result(result)
            return True
        return False


code_exceptions_mapping: Dict[int, Type[Exception]] = {
    1: InvalidVerifyKey,
    2: AccountNotFound,
    3: InvalidSession,
    4: UnVerifiedSession,
    5: UnknownTarget,
    6: FileNotFoundError,
    10: PermissionError,
    20: AccountMuted,
    30: MessageTooLong,
    400: InvalidArgument,
    500: RemoteException,
}


def validate_response(data: Dict[str, Any]) -> Union[dict, Exception]:
    """验证远程服务器的返回值

    Args:
        code (dict): 返回的对象

    Raises:
        Exception: 请参照 code_exceptions_mapping
    """
    if isinstance(data, dict):
        int_code: int = data.get("code")
    else:
        int_code = data
    if not isinstance(int_code, int) or int_code == 200 or int_code == 0:
        if "data" in data:
            return data["data"]
        return data
    exc_cls = code_exceptions_mapping.get(int_code)
    if exc_cls:
        return exc_cls(exc_cls.__doc__, data)
    return UnknownError(data)



"""Ariadne 基础的 parser, 包括 DetectPrefix 与 DetectSuffix"""
import abc
import fnmatch
import re
from typing import List, Optional, Type, Union

from graia.broadcast.entities.decorator import Decorator
from graia.broadcast.exceptions import ExecutionStop
from graia.broadcast.interfaces.decorator import DecoratorInterface

from ... import get_running
from ...event.message import GroupMessage
from ..chain import MessageChain
from ..element import At, Element, Plain, Quote, Source


class Compose(Decorator):
    """将多个基础 Decorator 串联起来"""

    def __init__(self, *deco: "ChainDecorator") -> None:
        self.deco: List[ChainDecorator] = list(deco)

    async def target(self, interface: DecoratorInterface):
        chain = await interface.dispatcher_interface.lookup_param("message_chain", MessageChain, None)
        for deco in self.deco:
            chain = await deco.decorate(chain, interface)
            if chain is None:
                break
        if interface.annotation is MessageChain:
            if chain is None:
                raise ExecutionStop
            return chain


class ChainDecorator(abc.ABC, Decorator):
    @abc.abstractmethod
    async def decorate(self, chain: MessageChain, interface: DecoratorInterface) -> Optional[MessageChain]:
        ...

    async def target(self, interface: DecoratorInterface):
        return await self.decorate(
            await interface.dispatcher_interface.lookup_param("message_chain", MessageChain, None), interface
        )


class DetectPrefix(ChainDecorator):
    """前缀检测器"""

    pre = True

    def __init__(self, prefix: str) -> None:
        """初始化前缀检测器.

        Args:
            prefix (str): 要匹配的前缀
        """
        self.prefix = prefix

    async def decorate(self, chain: MessageChain, interface: DecoratorInterface) -> Optional[MessageChain]:
        header = chain.include(Quote, Source)
        rest: MessageChain = chain.exclude(Quote, Source)
        if not rest.startswith(self.prefix):
            raise ExecutionStop
        result = rest.removeprefix(self.prefix).removeprefix(" ")
        if interface.annotation is MessageChain:
            return header + result


class DetectSuffix(ChainDecorator):
    """后缀检测器"""

    pre = True

    def __init__(self, suffix: str) -> None:
        """初始化后缀检测器.

        Args:
            suffix (str): 要匹配的后缀
        """
        self.suffix = suffix

    async def decorate(self, chain: MessageChain, interface: DecoratorInterface) -> Optional[MessageChain]:
        header = chain.include(Quote, Source)
        rest: MessageChain = chain.exclude(Quote, Source)
        if not rest.endswith(self.suffix):
            raise ExecutionStop
        result = rest.removesuffix(self.suffix).removesuffix(" ")
        if interface.annotation is MessageChain:
            return header + result


class MentionMe(ChainDecorator):
    """At 账号或者提到账号群昵称"""

    pre = True

    async def decorate(self, chain: MessageChain, interface: DecoratorInterface) -> Optional[MessageChain]:
        ariadne = get_running()
        if isinstance(interface.event, GroupMessage):
            name = (await ariadne.getMemberInfo(ariadne.account, interface.event.sender.group)).name
        else:
            name = (await ariadne.getBotProfile()).nickname
        header = chain.include(Quote, Source)
        rest: MessageChain = chain.exclude(Quote, Source)
        first: Element = rest[0]
        result: Optional[MessageChain] = None
        if rest and isinstance(first, Plain):
            if first.asDisplay().startswith(name):
                result = header + rest.removeprefix(name).removeprefix(" ")
        if rest and isinstance(first, At):
            if first.target == ariadne.account:
                result = header + MessageChain(rest.__root__[1:], inline=True).removeprefix(" ")

        if result is None:
            raise ExecutionStop
        if interface.annotation is MessageChain:
            return result


class Mention(ChainDecorator):
    """At 或提到指定人"""

    pre = True

    def __init__(self, target: Union[int, str]) -> None:
        self.person: Union[int, str] = target

    async def decorate(self, chain: MessageChain, interface: DecoratorInterface) -> Optional[MessageChain]:
        header = chain.include(Quote, Source)
        rest: MessageChain = chain.exclude(Quote, Source)
        first: Element = rest[0]
        result: Optional[MessageChain] = None
        if rest and isinstance(first, Plain):
            if isinstance(self.person, str) and first.asDisplay().startswith(self.person):
                result = header + rest.removeprefix(self.person).removeprefix(" ")
        if rest and isinstance(first, At):
            if isinstance(self.person, int) and first.target == self.person:
                result = header + MessageChain(rest.__root__[1:], inline=True).removeprefix(" ")

        if result is None:
            raise ExecutionStop
        if interface.annotation is MessageChain:
            return result


class ContainKeyword(ChainDecorator):
    """消息中含有指定关键字"""

    pre = True

    def __init__(self, keyword: str) -> None:
        self.keyword: str = keyword

    async def decorate(self, chain: MessageChain, interface: DecoratorInterface) -> Optional[MessageChain]:
        if self.keyword not in chain:
            raise ExecutionStop
        if interface.annotation is MessageChain:
            return chain


class MatchContent(ChainDecorator):
    """匹配字符串 / 消息链"""

    pre = True

    def __init__(self, content: Union[str, MessageChain]) -> None:
        self.content: Union[str, MessageChain] = content
        self.next: Optional[ChainDecorator] = None

    async def decorate(self, chain: MessageChain, interface: DecoratorInterface) -> Optional[MessageChain]:
        if isinstance(self.content, str) and chain.asDisplay() != self.content:
            raise ExecutionStop
        if isinstance(self.content, MessageChain) and chain != self.content:
            raise ExecutionStop
        if interface.annotation is MessageChain:
            return chain


class MatchRegex(ChainDecorator):
    """匹配正则表达式"""

    pre = True

    def __init__(self, regex: str, flags: re.RegexFlag = re.RegexFlag(0)) -> None:
        """初始化匹配正则表达式.

        Args:
            regex (str): 正则表达式
            flags (re.RegexFlag): 正则表达式标志
        """
        self.regex: str = regex
        self.flags: re.RegexFlag = flags

    async def decorate(self, chain: MessageChain, interface: DecoratorInterface) -> Optional[MessageChain]:
        if not re.match(self.regex, chain.asDisplay(), self.flags):
            raise ExecutionStop
        if interface.annotation is MessageChain:
            return chain


class MatchTemplate(ChainDecorator):
    """模板匹配"""

    def __init__(self, template: List[Union[Type[Element], Element]]) -> None:
        self.template: List[Union[Type[Element], Element, str]] = []
        for element in template:
            if isinstance(element, type) and element is not Plain:
                self.template.append(element)
            elif isinstance(element, Element) and not isinstance(element, Plain):
                self.template.append(element)
            else:
                element = element.text if isinstance(element, Plain) else "*"
                if self.template and isinstance(self.template[-1], str):
                    self.template[-1] = self.template[-1] + element
                else:
                    self.template.append(element)

    def match(self, chain: MessageChain):
        """匹配消息链"""
        chain = chain.asSendable()
        if len(self.template) != len(chain):
            return False
        for element, template in zip(chain, self.template):
            if isinstance(template, type) and not isinstance(element, template):
                return False
            elif isinstance(template, Element) and element != template:
                return False
            elif isinstance(template, str):
                if not isinstance(element, Plain) or not fnmatch.fnmatch(element.text, template):
                    return False
        return True

    async def decorate(self, chain: MessageChain, interface: DecoratorInterface) -> Optional[MessageChain]:
        if not self.match(chain):
            raise ExecutionStop
        if interface.annotation is MessageChain:
            return chain



"""反向 Adapter, 作为服务器让 mirai-api-http 连接"""

import asyncio
import json
from typing import Any, Dict, FrozenSet, Optional, Type, Union

from aiohttp import ClientSession, FormData
from fastapi import FastAPI, HTTPException, Request, WebSocket, WebSocketDisconnect
from graia.broadcast import Broadcast
from loguru import logger
from uvicorn import Config, Server

from graia.ariadne.adapter.forward import HttpAdapter

from ..model import CallMethod, DatetimeEncoder, MiraiSession
from ..util import await_predicate
from . import Adapter
from .util import SyncIDManager, validate_response


class NoSigServer(Server):
    """不注册 Signal 的服务器"""

    def install_signal_handlers(self) -> None:
        return


class ReverseAdapter(Adapter):
    """反向 Adapter 基类"""

    tags: FrozenSet[str] = frozenset(["reverse"])

    server: NoSigServer
    asgi: FastAPI
    mirai_session: MiraiSession
    broadcast: Broadcast
    session: Optional[ClientSession]

    def __init__(
        self,
        broadcast: Broadcast,
        mirai_session: MiraiSession,
        route: str = "/",
        log: bool = True,
        *,
        app: Optional[FastAPI] = None,
        port: int = 8000,
        server_cls: Type[NoSigServer] = NoSigServer,
        **config_kwargs: Any,
    ):
        """初始化 ReverseAdapter

        Args:
            broadcast (Broadcast): 事件系统
            mirai_session (MiraiSession): MiraiSession 实例
            route (str, optional): 服务路径. Defaults to "/".
            log (bool, optional): 是否启用连接日志. Defaults to True.
            app (Optional[FastAPI], optional): ASGI 应用. Defaults to None.
            port (int, optional): 服务端口. Defaults to 8000.
            server_cls (Type[NoSigServer], optional): Server 类. Defaults to NoSigServer.
            **config_kwargs (Any, optional): 额外配置参数. Defaults to {}.
        """
        super().__init__(broadcast, mirai_session)
        self.asgi = app or FastAPI()
        self.route = route
        LOG_CONFIG = {
            "version": 1,
            "disable_existing_loggers": False,
            "handlers": {
                "default": {
                    "class": "graia.ariadne.util.LoguruHandler",
                },
            },
            "loggers": {
                "uvicorn.error": {"handlers": ["default"] if log else [], "level": "INFO"},
                "uvicorn.access": {"handlers": ["default"] if log else [], "level": "INFO"},
            },
        }
        self.server = server_cls(Config(self.asgi, port=port, log_config=LOG_CONFIG, **config_kwargs))

    async def stop(self) -> None:
        """停止服务器"""
        self.running = False
        self.server.should_exit = True
        try:
            await asyncio.wait_for(self.fetch_task, timeout=5.0)
        except asyncio.TimeoutError:
            self.server.force_exit = True
        await self.fetch_task
        self.fetch_task = None

    async def fetch_cycle(self):
        async with ClientSession() as session:
            self.session = session
            await self.server.serve()
        self.session = None


class ComposeWebhookAdapter(ReverseAdapter):
    """Webhook (反向 HTTP) Adapter, 同时使用了正向 HTTP 以进行 API 调用支持"""

    tags: FrozenSet[str] = frozenset(["reverse", "http"])

    def __init__(
        self,
        broadcast: Broadcast,
        mirai_session: MiraiSession,
        route: str = "/",
        extra_headers: Optional[Dict[str, str]] = None,
        log: bool = True,
        *,
        app: Optional[FastAPI] = None,
        port: int = 8000,
        server_cls: Type[NoSigServer] = NoSigServer,
        **config_kwargs: Any,
    ):
        """初始化 ComposeWebhookAdapter

        Args:
            broadcast (Broadcast): 事件系统
            mirai_session (MiraiSession): MiraiSession 实例
            route (str, optional): 服务路径. Defaults to "/".
            extra_headers (Optional[Dict[str, str]], optional): 额外的请求头. Defaults to None.
            log (bool, optional): 是否启用连接日志. Defaults to True.
            app (Optional[FastAPI], optional): ASGI 应用. Defaults to None.
            port (int, optional): 服务端口. Defaults to 8000.
            server_cls (Type[NoSigServer], optional): Server 类. Defaults to NoSigServer.
            **config_kwargs (Any, optional): 额外配置参数. Defaults to {}.
        """
        super().__init__(
            broadcast,
            mirai_session,
            route,
            log,
            app=app,
            port=port,
            server_cls=server_cls,
            **config_kwargs,
        )
        self.asgi.add_api_route(self.route, self.http_endpoint, methods=["POST"])
        self.extra_headers: Dict[str, str] = extra_headers or {}
        self.connected: bool = False

    async def http_endpoint(self, request: Request):
        header: Dict[str, str] = dict(request.headers.items())
        if header["qq"] == str(self.mirai_session.account):
            for key, val in self.extra_headers.items():
                key = key.lower()
                if val != header.get(key, ""):
                    raise HTTPException(status_code=401, detail="Authorization Failed")
            self.connected = True
            await self.event_queue.put(self.build_event(await request.json()))
        return {"command": "", "data": {}}

    authenticate = HttpAdapter.authenticate

    async def call_api(
        self,
        action: str,
        method: CallMethod,
        data: Optional[Union[Dict[str, Any], str, FormData]] = None,
    ) -> Union[dict, list]:
        await await_predicate(lambda: self.connected)
        return await HttpAdapter.call_api(self, action, method, data)


class ReverseWebsocketAdapter(ReverseAdapter):
    """反向 WebSocket Adapter"""

    tags: FrozenSet[str] = frozenset(["reverse", "websocket"])

    def __init__(
        self,
        broadcast: Broadcast,
        mirai_session: MiraiSession,
        route: str = "/",
        extra_headers: Optional[Dict[str, str]] = None,
        query_params: Optional[Dict[str, str]] = None,
        log: bool = True,
        *,
        app: Optional[FastAPI] = None,
        port: int = 8000,
        server_cls: Type[NoSigServer] = NoSigServer,
        **config_kwargs: Any,
    ):
        """初始化 ReverseWebsocketAdapter

        Args:
            broadcast (Broadcast): 事件系统
            mirai_session (MiraiSession): MiraiSession 实例
            route (str, optional): 服务路径. Defaults to "/".
            extra_headers (Optional[Dict[str, str]], optional): 额外的请求头. Defaults to None.
            query_params (Optional[Dict[str, str]], optional): 额外的请求参数 (在 url 中). Defaults to None.
            log (bool, optional): 是否启用连接日志. Defaults to True.
            app (Optional[FastAPI], optional): ASGI 应用. Defaults to None.
            port (int, optional): 服务端口. Defaults to 8000.
            server_cls (Type[NoSigServer], optional): Server 类. Defaults to NoSigServer.
            **config_kwargs (Any, optional): 额外配置参数. Defaults to {}.
        """
        super().__init__(
            broadcast,
            mirai_session,
            route,
            log,
            app=app,
            port=port,
            server_cls=server_cls,
            **config_kwargs,
        )
        self.asgi.add_api_websocket_route(self.route, self.websocket_endpoint)
        self.id_manager = SyncIDManager()
        self.websocket: Optional[WebSocket] = None
        self.extra_headers: Dict[str, str] = extra_headers or {}
        self.query_params: Dict[str, str] = query_params or {}
        self.connected: bool = False

    async def websocket_endpoint(self, websocket: WebSocket):
        header: Dict[str, str] = dict(websocket.headers.items())
        query_params: Dict[str, str] = dict(websocket.query_params.items())
        for key, val in self.extra_headers.items():
            key = key.lower()
            if val != header.get(key, ""):
                raise HTTPException(status_code=401, detail="Authorization Failed")
        for key, val in self.query_params.items():
            if val != query_params.get(key, ""):
                raise HTTPException(status_code=401, detail="Authorization Failed")
        await websocket.accept()
        self.websocket = websocket
        try:
            asyncio.create_task(self.get_session_key())
            while True:
                raw_data = await websocket.receive_json()
                sync_id: int = int(raw_data["syncId"] or -1)
                data: dict = raw_data["data"]
                if not self.id_manager.free(sync_id, validate_response(data)):
                    await self.event_queue.put(self.build_event(data))
                    self.connected = True
        except WebSocketDisconnect:
            self.websocket = None
            self.mirai_session.session_key = None
            self.connected = False

    async def get_session_key(self):
        if not self.mirai_session.single_mode and not self.mirai_session.session_key:
            future = self.broadcast.loop.create_future()
            sync_id: int = self.id_manager.allocate(future)
            content = {
                "syncId": str(sync_id),
                "command": "verify",
                "content": {
                    "verifyKey": self.mirai_session.verify_key,
                    "qq": self.mirai_session.account,
                    "sessionKey": None,
                },
            }
            await self.websocket.send_text(json.dumps(content))
            self.mirai_session.session_key = (await future)["session"]
            logger.success("Successfully got session key")

    async def call_api(
        self,
        action: str,
        method: CallMethod,
        data: Optional[Union[Dict[str, Any], str, FormData]] = None,
    ) -> Union[dict, list]:
        await await_predicate(lambda: self.connected)
        await await_predicate(lambda: self.websocket is not None and self.mirai_session.session_key)
        future = self.broadcast.loop.create_future()
        sync_id: int = self.id_manager.allocate(future)
        content = {
            "syncId": str(sync_id),
            "command": action.replace("/", "_"),
            "content": data,
        }
        if method == CallMethod.RESTGET:
            content["subCommand"] = "get"
        elif method == CallMethod.RESTPOST:
            content["subCommand"] = "update"
        elif method == CallMethod.MULTIPART:
            self.id_manager.free(
                sync_id,
                NotImplementedError(f"Unsupported operation for ReverseWebsocketAdapter: {method}"),
            )
        await self.websocket.send_text(json.dumps(content, cls=DatetimeEncoder))


class ComposeReverseWebsocketAdapter(ReverseWebsocketAdapter):
    """反向 WebSocket 与正向 HTTP 的组合 Adapter"""

    tags: FrozenSet[str] = frozenset(["reverse", "websocket", "http"])

    async def call_api(
        self, action: str, method: CallMethod, data: Optional[Union[Dict[str, Any], str, FormData]] = None
    ) -> Union[dict, list]:
        await await_predicate(lambda: self.connected)
        return await HttpAdapter.call_api(self, action, method, data)



"""Ariadne

一个优雅的 QQ Bot 框架.
"""
from typing import Type

import graia.ariadne.event.lifecycle  # noqa: F401
import graia.ariadne.event.message  # noqa: F401
import graia.ariadne.event.mirai  # noqa: F401

from .app import Ariadne
from .typing import T


def get_running(type: Type[T] = Ariadne) -> T:
    """获取正在运行的实例

    Args:
        type (Type[T]): 实例类型

    Returns:
        T: 对应类型实例
    """
    from asyncio import AbstractEventLoop

    from graia.broadcast import Broadcast

    from .adapter import Adapter
    from .context import context_map

    if type in {Adapter, Ariadne, Broadcast, AbstractEventLoop}:
        if val := context_map.get(type.__name__).get(None):
            return val
    for ariadne_inst in Ariadne.running:
        if type in ariadne_inst.info:
            return ariadne_inst.info[type]
